@@ -338,6 +338,15 @@ class Type(
                 ~Eval('comparison', False)),
             ]
 
+    @classmethod
+    def copy(cls, types, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('template', None)
+        return super().copy(types, default=default)
+
     @classmethod
     def delete(cls, types):
         types = cls.search([
